import numpy as np
import pandas as pd
from numpy import ndarray
from vteklib.regressions.regression import Regression
from vteklib.regressions.poly import Poly
from pandas import Series


class PlotData:
    """
    A data class for plot params.
    """
    def __init__(self,
                 X: ndarray | str,
                 Y: ndarray | str,
                 title: str = 'plot by MikeP',
                 label: str = 'y=f(x)',
                 x_name: str = 'x',
                 y_name: str = 'y',
                 x_error: ndarray | None = None,
                 y_error: ndarray | None = None
                 ):

        if x_error is None:
            x_error = np.zeros(len(X))
        if y_error is None:
            y_error = np.zeros(len(Y))
        self.df = pd.DataFrame()
        self.df[x_name] = format_input(X)
        self.df[y_name] = format_input(Y)
        self.df['x_error'] = x_error
        self.df['y_error'] = y_error
        self.x_name = x_name
        self.y_name = y_name
        self.title = title
        self.label = label
        self.approximated: bool = False
        self.theoretical: bool = False
        self.reg = None
        self.approx_name = ''
        self.x_test_range = [np.min(self.df[self.x_name]), np.max(self.df[self.x_name])]
        self.num_of_pts = len(X)

    def build_theoretical_curve(self, res):
        self.df['x_theoretical'] = self.df[self.x_name]
        self.df['y_theoretical'] = res
        self.theoretical = True

    def approximate(self,
                    reg: Regression,
                    x_train: None | Series = None,
                    y_train: None | Series = None,
                    x_test_range=None,
                    repr_equation: bool = False):
        """
        Approximates X and Y plot data with chosen regression.
        Linear and Polynomial regressions is fully supported.
        Creates an additional column in PlotData.df with predicted values & changes self.approximated flag to True
        """
        if x_test_range is not None:
            self.x_test_range = x_test_range
        if x_train is None:
            x_train = self.df[self.x_name]
        if y_train is None:
            y_train = self.df[self.y_name]

        if not self.approximated:
            reg.fit(x_train, y_train)
            self.approx_name: str = f"{reg} {self.y_name}({self.x_name})"
            self.approximated = True
            if repr_equation:
                self.label = f"{self.label}\n {reg.equation}"
            self.reg = reg
            return reg

    def __repr__(self):
        return self.df.__str__()


def format_input(args: str | ndarray) -> ndarray | list[ndarray]:
    if type(args) == ndarray:
        return args
    return np.array(args.replace(',', '.').split(), dtype=float)


def test_figures():
    Y = format_input("""110280
        107030
        104140
        101660
        99330""")
    X = format_input("""36
        37
        38
        39
        40""")
    title = 'График зависимости давления от объема в изотермическом процессе'
    label = 'T=const'
    x_title = 'Объём, мл'
    y_title = 'Давление, гПа'
    plot_data = PlotData(X, Y,
                         title=title,
                         label=label,
                         x_name=x_title,
                         y_name=y_title)
    print(type(plot_data.df[plot_data.x_name]))
    plot_data.approximate(Poly())
    print(plot_data)


if __name__ == '__main__':
    test_figures()



from abc import ABC, abstractmethod
from pandas import Series
from numpy import ndarray


class Regression(ABC):
    """
    An abstract base class for regression.
    You can inherit this class for specific regression type.
    In this case overloading abstract methods required.
    """
    @abstractmethod
    def fit(self, x_data: Series, y_data: Series):
        """
        Trains regression model on given dataset
        """
        ...

    @abstractmethod
    def predict(self, x_data: Series) -> ndarray:
        """
        Returns ndarray with predicted values
        """
        ...



from setuptools import setup


setup(
    name='vteklib',
    version='0.3.0',
    packages=['vteklib', 'vteklib.utils', 'vteklib.regressions'],
    install_requires=['numpy', 'matplotlib', 'pandas', 'sklearn', 'scipy', 'openpyxl'],
    url='https://github.com/mixx3/vteklib',
    license='BSD 2-Clause "Simplified"',
    author='Mike Parfenov',
    author_email='mmikee00800@gmail.com',
    description='Wrapper around matplotlib for easier 2D approximation & scientific-style plots'
)



import json

import numpy as np
import pandas
import pandas as pd
from collections import namedtuple
from functools import lru_cache


class ExcelFile:
    def __init__(self, excel_fp: str):
        self.df: pd.DataFrame = pandas.read_excel(excel_fp)
        self.all_series: list[DataSeries] = []

    def get_series(self):
        for col in self.df.columns:
            series_in_row: list[DataSeries] = []
            for i in self.df[col]:
                if type(i) == str:
                    s = DataSeries()
                    series_in_row.append(s)
                    s.name = i
                    s.data = []
                elif not pd.isna(i) and type(i) != str:
                    if len(series_in_row):
                        series_in_row[-1].data.append(i)
                    else:
                        continue
            for series in series_in_row:
                if len(series.data):
                    self.all_series.append(series)
        return self.all_series

    def to_json(self):
        names = dict()
        res = dict()
        for s in self.get_series():
            if s.name not in names.keys():
                res[s.name] = s.data
                names[s.name] = 1
            else:
                res[f"{s.name}({names[s.name]})"] = s.data
                names[s.name] += 1
        return json.dumps(res, ensure_ascii=False, indent=4)


class DataSeries:
    __slots__ = ('name', 'data')
    name: str
    data: list

    def __init__(self):
        pass

    def __str__(self):
        return f"{self.name} {self.data}"


@lru_cache(None)
def get_data(path: str):
    ef = ExcelFile(path)
    series = ef.to_json()
    return series
